"""
A general library for working with DTD documentation files.

$Id: dddlib.py,v 1.2 2000/01/06 14:02:43 larsga Exp $
"""

import ezsax

from xml.parsers.xmlproc import utils,xmldtd,dtdparser

# --- Loading DTD

def load_dtd(dtdurl):
    dp=dtdparser.DTDParser()
    dp.set_error_handler(utils.ErrorPrinter(dp))
    dtd=xmldtd.CompleteDTD(dp)
    #dtd.compile_content_models(0)
    dp.set_dtd_consumer(dtd)
    dp.parse_resource(dtdurl)

    return dtd

# --- Loading DTD documentation

class DTDDocumentation:

    def __init__(self):
        self.root_elems=[]
        self.elems={}
        self.attrgrps = {}
        self.version=None
        self.pubid=None
        self.name=None
        self.descr=None
        self.shortname=None
        self.creators = []

    def get_elements(self):
        return self.elems.values()

class Creator:

    def __init__(self, name = None, email = None, homep = None):
        self.name = name
        self.email = email
        self.homep = homep
    
class ElementDocumentation:

    # attributes: name, one_line, descr
    
    def __init__(self,name = None):
        self.name = name
        self.one_line = None
        self.descr = None
        self.__attrs=[]

    def add_attribute(self,attr):
        self.__attrs.append(attr)

    def get_attributes(self):
        return self.__attrs
        
    def get_attribute(self,name):
        for attr in self.__attrs:
            if attr.name==name:
                return attr

        if options.strict:
            raise KeyError("Undocumented attribute '%s' on element '%s'" %
                           (name,self.name))
        else:
            return AttributeDocumentation(name,"")

    def get_attribute_count(self):
        return len(self.__attrs)

class AttributeGroup:

    def __init__(self):
        self.name = None
        self.attrs = []

    def add_attribute(self,attr):
        self.attrs.append(attr)

    def get_attributes(self):
        return self.attrs
        
class AttributeDocumentation:

    # attributes: name, descr
    
    def __init__(self,name=None,descr=None):
        self.name=name
        self.descr=descr
    
class DTDDocHandler(ezsax.DocHandler):

    def __init__(self,dtd):
        ezsax.DocHandler.__init__(self)
        self.dtd=dtd
        self.dtddoc=DTDDocumentation()
        self.curelem=ElementDocumentation()
        self.curattr=AttributeDocumentation()
        self.curattrgrp=AttributeGroup()
        self.creator = Creator()
        self.descr=[]
    
    def startElement(self,name,attrs):
        if name=="elemref":
            self.descr.append(self.text)

    def endElement(self,name,attrs):
        # Header        
        if name=="name" and self.element_stack[-1][0]=="dtd-info":
            self.dtddoc.name=self.text
        if name=="descr" and self.element_stack[-1][0]=="dtd-info":
            self.descr.append(self.text)
            self.dtddoc.descr=self.descr
            self.descr=[]
        elif name=="version":
            self.dtddoc.version=self.text
        elif name=="pubid":
            self.dtddoc.pubid=self.text
        elif name=="name" and self.element_stack[-1][0]=="root-elements":
            try:
                self.dtd.get_elem(self.text)
            except KeyError:
                print "ERROR: Reference to undeclared element '%s'" % self.text
                
            self.dtddoc.root_elems.append(self.text)

        # Elements
            
        elif name=="element":
            self.dtddoc.elems[self.curelem.name]=self.curelem
            self.curelem=ElementDocumentation()
        elif name=="name" and self.element_stack[-1][0]=="element":
            self.curelem.name=self.text

        # Attributes

        elif name=="attribute":
            self.curattr.descr=self.descr
            self.descr=[]
            if self.element_stack[-1][0]=="element":
                self.curelem.add_attribute(self.curattr)
            else:
                self.curattrgrp.add_attribute(self.curattr)
            self.curattr=AttributeDocumentation()            
        elif name=="name" and self.element_stack[-1][0]=="attribute":
            self.curattr.name=self.text

        # Description

        elif name=="descr":
            self.descr.append(self.text)
            if self.element_stack[-1][0] == "element":
                self.curelem.descr=self.descr
                self.descr=[]
        elif name=="one-line":
            if self.element_stack[-2][0] == "element":
                self.curelem.one_line=self.text
            self.descr.append(self.text)
        elif name=="elemref":
            try:
                self.dtd.get_elem(self.text)
            except KeyError:
                print "ERROR: Reference to undeclared element '%s'" % self.text

            self.descr.append(("elemref",self.text))

        # Attribute group

        elif name == "name" and self.element_stack[-1][0] == "attribute-group":
            self.curattrgrp.name = self.text
        elif name == "attribute-group":
            self.dtddoc.attrgrps[self.curattrgrp.name] = self.curattrgrp
            self.curattrgrp = AttributeGroup()
        elif name == "attribute-group-ref":
            try:
                attrgrp = self.dtddoc.attrgrps[self.text]
            except KeyError:
                print "ERROR: Reference to undeclared attribute group '%s'" % self.text
                return

            for attr in attrgrp.get_attributes():
                self.curelem.add_attribute(attr)

        # Creator

        elif name == "creator":
            self.dtddoc.creators.append(self.creator)
            self.creator = Creator()
        elif name=="name" and self.element_stack[-1][0]=="creator":
            self.creator.name=self.text        
        elif name=="email" and self.element_stack[-1][0]=="creator":
            self.creator.email=self.text
        elif name=="homep" and self.element_stack[-1][0]=="creator":
            self.creator.homep=self.text        
                
# --- Loading and dumping
            
def parse_dtddoc(dtd,dtddocurl):
    handler=DTDDocHandler(dtd)
    ezsax.parse(handler,dtddocurl)
    return handler.dtddoc

def dump_dtddoc(outfile, doco):
    out = open(outfile, "w")
    out.write('<!DOCTYPE dtddoc PUBLIC "-//LM Garshol//DTD Documentation DTD 0.1//EN" "dtddoc.dtd">\n')
    out.write("<dtddoc>\n")
    out.write("  <dtd-info>\n")
    out.write("    <name>%s</name>\n" % doco.name)
    out.write("    <shortname>%s</shortname>\n" % doco.shortname)
    out.write("    <version>%s</version>\n" % doco.version)
    out.write("    <pubid>%s</pubid>\n" % doco.pubid)
    out.write("\n")
    for creator in doco.creators:
        out.write("    <creator>\n")
        out.write("      <name>%s</name>\n" % doco.creator.name)
        out.write("      <email>%s</email>\n" % doco.creator.email)
        out.write("      <homep>%s</homep>\n" % doco.creator.homep)
        out.write("    </creator>\n")
    out.write("\n")
    out.write("    <descr>\n")
    out.write("    </descr>\n")
    out.write("  </dtd-info>\n")
    out.write("\n")
    out.write("  <root-elements>\n")    
    out.write("    <name></name>\n")    
    out.write("  </root-elements>\n")    
    out.write("\n")

    for elem in doco.get_elements():
        out.write("  <element>\n")
        out.write("    <name>%s</name>\n" % elem.name)
        out.write("    <descr><one-line></one-line></descr>\n")

        for attr in elem.get_attributes():
            out.write("\n    <attribute>\n")
            out.write("      <name>%s</name>\n" % attr.name)
            out.write("      <descr><one-line></one-line></descr>\n")
            out.write("    </attribute>\n")
        
        out.write("  </element>\n\n")

    out.write("</dtddoc>\n")
    out.close()
