"""A DTD documentation generator.

$Id: dtddoc.py,v 1.14 2001/08/07 21:30:34 larsga Exp $
"""

# Todo:
# - test on some serious DTDs
# - more options
#   - add a -r <path> option to enable specifying the root path
#   - add a --split option to split doco into separate files
#   - add option for reverse-engineering of attribute groups
# - extend DTD
#   - allow global documentation of attributes
#   - include notations?
#   - include entities?
# - more functionality
#   - support catalog files
#   - backend for DocBook RefEntry documentation
#     - fix references to other refentries
#     - intro part: table of DTD properties
#     - show attributes
#     - sort elements
#     - root elem info, elem ix, attr ix
#   - backend for complete XML documentation
#   - support for turning complete XML documentation into PDF etc

import sys

import ezsax
from xml.parsers.xmlproc import xmlapp, xmlproc

if xmlproc.version<"0.61":
    print "dtddoc requires xmlproc 0.61 or newer."
    print "Loaded "+xmlproc.version+" from "+xmlproc.__file__
    sys.exit(1)

import htmldoc, refentry, cmdarg, dddlib, xtmdoc

version = "0.13"

# --- Configuration

class Options:

    def __init__(self):
        self.format="html"
        self.attr_index=1
        self.elem_index=1
        self.elem_index_one_line=1
        self.silent=0
        self.css="dtd.css"
        self.strict=0

# --- DTD loading and processing

def traverse_cm(cur,cm,parents):
    if cm==None:
        return

    for item in cm[1]:
        if len(item)==2:
            try:
                parents[item[0]][cur]=1
            except KeyError:
                print "ERROR: Undeclared element '%s'" % item[0]
        else:
            traverse_cm(cur,item,parents)

def parse_dtd(dtdurl):
    dtd = dddlib.load_dtd(dtdurl)

    # Processing the DTD

    if not options.silent:
        print "Processing DTD"

    parents={}
    parents["#PCDATA"]={}
    for elem_name in dtd.get_elements():
        parents[elem_name]={}    

    for elem_name in dtd.get_elements():
        elem=dtd.get_elem(elem_name)
        traverse_cm(elem_name,elem.get_content_model(),parents)

    return (dtd,parents)

# --- Command-line handlers

def print_usage():
    print
    print "  dtddoc.py [opts] -f <format> dtdfile docofile outputfile"
    print
    print "  Options:"
    print "  -e:          Don't make an element index"
    print "  -o:          Skip one-line element descriptions in element index"
    print "  -a:          Don't make an attribute index"
    print "  -s:          Be silent"
    print "  -t:          Be strict (require everything to be documented)"
    print "  -f <format>: Output documentation in <format> (html, ref, xtm)"
    print
    print "  Required arguments:"
    print "  dtdfile:    The DTD"
    print "  docofile:   XML documentation for the DTD"
    print "  outputfile: Output destination"

def error(msg):
    print
    print "ERROR: %s!" % msg
    print_usage()
    sys.exit(1)    

# --- The program itself

def run_dtddoc(dtdurl, dtddoc, outfile, opts):
    global options
    options = opts           # make visible to other functions
    dddlib.options = options # make visible in dddlib as well    
    
    try:
        # Loading

        if not options.silent:
            print "Parsing DTD"
        (dtd,parents)=parse_dtd(dtdurl)

        if not options.silent:
            print "Parsing documentation"
        dtddoc=dddlib.parse_dtddoc(dtd,dtddoc)

        # Printing documentation

        if not options.silent:
            print "Printing documentation"

        if not hasattr(dtddoc,"name"):
            print "ERROR: DTD has no name. Aborting."
            sys.exit(1)

        out=open(outfile,"w")
        backends[options.format].write_doco(out,dtddoc,dtd,parents,options)

    except KeyError,e:
        print "ERROR: "+str(e)
        print "Aborting"
        sys.exit(1)
    
# --- Main program

backends = {"html" : htmldoc, "ref" : refentry, "xtm" : xtmdoc}

if __name__ == "__main__":
    # Initialization

    options = Options()

    # Interpreting command-line arguments

    try:
        opts = cmdarg.process(sys.argv[1:], "f:eaost", [], ["-f"])

        if len(opts.get_remainder()) != 3:
            error("Need exactly three arguments")

    except cmdarg.ArgumentException, e:
        error(e)

    options.elem_index          = not opts.has_opt("-e")
    options.elem_index_one_line = not opts.has_opt("-o")
    options.attr_index          = not opts.has_opt("-a")
    options.silent              = opts.has_opt("-s")
    options.format              = opts.get_opt_value("-f")
    options.strict              = opts.has_opt("-t")

    if not backends.has_key(options.format):
        error("Unsupported format '%s'" % options.format)

    dtdurl  = opts.get_remainder()[0]
    dtddoc  = opts.get_remainder()[1]
    outfile = opts.get_remainder()[2]

    run_dtddoc(dtdurl, dtddoc, outfile, options)
