
from xml.sax import saxlib,saxexts,saxutils
import sys

class SAXAdapter(saxlib.DocumentHandler):

    def __init__(self,dh):
        self.dh=dh
        self.tree=None
        # add hash of PI targets which should have their data parsed
        # as pseudo-attributes?
    
    def startElement(self,name,attrs):            
        if self.tree!=None:
            if self.dh.text!="":
                self.tree.append(self.dh.text)
                
            next=[name,attrs]
            self.tree.append(next)
            self.level_stack.append(self.tree)
            self.tree=next
            self.dh.text=""
        else:
            self.dh.startElement(name,attrs)
            self.dh.text=""
            self.dh.element_stack.append((name,attrs))  # copy attrs??

    def characters(self,data,start,len):
        self.dh.text=self.dh.text+data[start:start+len]

    def endElement(self,name):
        if self.tree!=None:
            if self.dh.text!="":
                self.tree.append(self.dh.text)
            self.tree=self.level_stack[-1]
            del self.level_stack[-1]

            if self.tree=="#root":
                self.tree=None

        if self.tree==None:
            attrs=self.dh.element_stack[-1][1]
            del self.dh.element_stack[-1]
            self.dh.endElement(name,attrs)
            
        self.dh.text=""
        
    def processingInstruction(self,target,data):
        self.dh.processingInstruction(target,data)

    def start_building(self,root):
        self.tree=root
        self.level_stack=["#root"]
        
class DocHandler:

    def __init__(self):
        self.element_stack=[]  # stack of (name,attrs) tuples
        self.locator=None      # locator, if any
        self.text=""           # text seen so far in current element
                               # (reset whenever a tag is seen)
        self.adapter=None      # set by the parse function; if you don't
                               # use it you must set it yourself
    
    def startElement(self,name,attrs):
        pass

    def endElement(self,name,attrs):
        pass

    def processingInstruction(self,target,data):
        pass

    def build_tree(self):
        root=[]
        self.adapter.start_building(root)
        return root

def parse(dochandler,sysid,err=None):
    p=saxexts.make_parser()
    if err==None:
        err=saxutils.ErrorPrinter()
    p.setErrorHandler(err)
    
    adapter=SAXAdapter(dochandler)
    p.setDocumentHandler(adapter)
    dochandler.adapter=adapter
    
    p.parse(sysid)
    
# --- Main program

#parse(DocHandler(),sys.argv[1])
