"""A dtddoc backend that produces a topic map in XTM format as output.

$Id: xtmdoc.py,v 1.2 2001/08/07 21:32:23 larsga Exp $
"""

import types, sys

try:
    from net.ontopia.topicmaps.tmproc import tm, tmutils
    tmproc_installed = 1
except ImportError, e:
    tmproc_installed = 0

if hasattr(types, "UnicodeType"): # hack to make it work with Py 1.5.x and 2.0
    utype = types.UnicodeType
else:
    utype = types.StringType # just a dummy value

import docutil


def make_name(base, sortname = None):
    if sortname:
        return tm.TopicName((), (tm.Name(base),), (), (tm.Name(sortname), ))
    else:
        return tm.TopicName((), (tm.Name(base),))

def load_tm(file, name):
    imp = tmutils.TMImporter(name, tm, {})
    tm_utils = tmutils.TMUtils(imp)
    return tm_utils.parse(tm_utils.get_parser(), file)

def write_doco(out, dtddoc, dtd, parents, options):
    if not tmproc_installed:
        sys.err.write("ERROR: tmproc not installed!\n")
        return
    
    topicmap = load_tm("basetm.iso", "xtmdoc")
    type_dtd = topicmap.get_topic_by_sgml_id("dtd")
    type_elem = topicmap.get_topic_by_sgml_id("element-type")
    type_attr = topicmap.get_topic_by_sgml_id("attribute-type")
    assoc_child = topicmap.get_topic_by_sgml_id("child-of")
    assoc_docelem = topicmap.get_topic_by_sgml_id("document-element-of")
    assoc_attr = topicmap.get_topic_by_sgml_id("attribute-appears-on")
    role_child = topicmap.get_topic_by_sgml_id("child")
    role_parent = topicmap.get_topic_by_sgml_id("parent")
    occ_desc = topicmap.get_topic_by_sgml_id("brief-description")

    dtd_topic = tm.Topic("the-dtd")
    dtd_topic.add_type(type_dtd)
    dtd_topic.add_name(make_name(dtddoc.name))
    topicmap.add_topic(dtd_topic)

    # creating element topics
    for elem_name in dtd.get_elements():
        elem = dtd.get_elem(elem_name)
        topic = tm.Topic("el-" + elem_name)
        topic.add_type(type_elem)
        topic.add_name(make_name(elem_name))
        topicmap.add_topic(topic)

        if dtddoc.elems.has_key(elem_name):
            one_line = dtddoc.elems[elem_name].one_line
            occ = tm.Occurrence("", occ_desc)
            occ.set_value(one_line)
            topic.add_occurrence(occ)
        
        for attr_name in elem.get_attr_list():
            attr = elem.get_attr(attr_name)

            attr_topic = topicmap.get_topic_by_sgml_id("attr-" + attr_name)
            if attr_topic == None:
                attr_topic = tm.Topic("attr-" + attr_name)
                attr_topic.add_type(type_attr)
                attr_topic.add_name(make_name(attr_name))
                topicmap.add_topic(attr_topic)

            assoc = tm.Association("", assoc_attr)
            assoc.add_role(tm.AssociationRole("", type_elem, (topic,)))
            assoc.add_role(tm.AssociationRole("", type_attr, (attr_topic,)))
            topicmap.add_association(assoc)            

    # child-parent associations
    for elem_name in dtd.get_elements():
        elem = dtd.get_elem(elem_name)
        child = topicmap.get_topic_by_sgml_id("el-" + elem_name)

        for parent in parents[elem_name].keys():
            parent = topicmap.get_topic_by_sgml_id("el-" + parent)
            
            assoc = tm.Association("", assoc_child)
            assoc.add_role(tm.AssociationRole("", role_child, (child,)))
            assoc.add_role(tm.AssociationRole("", role_parent, (parent,)))
            topicmap.add_association(assoc)

    # document element associations
    for elem_name in dtddoc.root_elems:
        child = topicmap.get_topic_by_sgml_id("el-" + elem_name)
        if child == None:
            print "ERROR: Non-existent root element " + repr(elem_name)
            continue

        assoc = tm.Association("", assoc_docelem)
        assoc.add_role(tm.AssociationRole("", type_elem, (child,)))
        assoc.add_role(tm.AssociationRole("", type_dtd, (dtd_topic,)))
        topicmap.add_association(assoc)        
    
    # export topic map
    from xml.sax.writer import XmlWriter
    p = XmlWriter(out)
    exp = tmutils.XTMExporter().export(topicmap, p)
    p.endDocument()
    out.close()
