#!/usr/bin/env python

# $Id: tmproc.py,v 1.5 2000/10/03 09:25:27 grove Exp $

# ----------------------------------------------------------------------------
#
# tmproc - a Topic Map engine
#
# Copyright (C) 1999, 2000 Geir O. Grnmo <grove@ontopia.net>
#
# ----------------------------------------------------------------------------

"""A command line tool for the topic map engine."""

__author__ = "Geir Ove Grnmo, <grove@ontopia.net>"
__version__ = "0.21"

# ============================================================================
# Imported modules
# ============================================================================

from tmutils import *
import string, sys, getopt

# ============================================================================
# Functions
# ============================================================================

def usage (str):

    line = "------------------------------------------------------------------------------"
    usage = """%s
tmproc version %s
%s

  Usage: python -i tmproc.py [options] file.xml

  Options:

     --validate : use a validating SGML/XML parser
     --parser   : use the specified parser (e.g. 'xml.sax.drivers.drv_xmlproc')
     --debug    : write debug messages to stdout
     --arch=?   : do architectural processing on the file, where ? is 
                  the architecture name
     --name     : the name of the topic map
     --test     : dump information about the topic map to stdout
     --module=? : load the topic map classes from the specified python 
                  module [default: tm]
"""

    print usage % (line, __version__, line)
    sys.exit(1)

# =============================================================================
# Main program
# =============================================================================

# ----------------------------------------------------------------------------
# Initialize variables

validate = 0
driver=None
debug = 1
test = 0
grove = 0
arch_name = None

tm_name = "Default topic map"
tm_module = "tm"
tm_module_keywords = {}

# ----------------------------------------------------------------------------
# Parse command line options

try:
    # Read options from command line
    optlist, args = getopt.getopt(sys.argv[1:], "", ["debug", "validate", "test", "arch=", "name=", "module=", "parser=", "grove"])

    if len(args) != 1:
	usage("I need at least two arguments.")

    for (arg, val) in optlist:
	if arg=="--validate": # Use validating parser?
	    validate = 1
	elif arg=="--parser": # Use specific parser
	    driver=val
	elif arg=="--debug":  # Turn debugging information on
	    debug = 1
	elif arg=="--arch":   # Perform architectural processing
	    arch_name = val
	elif arg=="--name":   # Give the Topic Map a name
	    tm_name = val
	elif arg=="--test":   # Dump Topic Map information to stdout
	    test = 1
	elif arg=="--grove":  # Internal test
	    grove = 1
	elif arg=="--module": # Use specific Topic Map module implementation
	    tm_module=val
	    if tm_module == "tmdb":
		import gadfly
		tm_module_keywords = {"connection": gadfly.gadfly("tm", "tmdata")}
	    
# Catch getopt errors and display usage information
except getopt.error, e:
	usage(e)

# ----------------------------------------------------------------------------
# Create import handler and utility object

tm_handler = TMImporter(tm_name, tm_module, tm_module_keywords)
tm_utils = TMUtils(tm_handler)

# ----------------------------------------------------------------------------
# Initialize architectural processing

if arch_name:
    from xml.arch import xmlarch
    tm_utils.set_arch_handler(arch_name, xmlarch.ArchDocHandler())
    
# ----------------------------------------------------------------------------
# Create parser object

if driver:
    parser = tm_utils.get_parser_by_driver(driver)
else:
    parser = tm_utils.get_parser(validate)

# ----------------------------------------------------------------------------
# Read topic map document

if debug:
    print "Starting topic map processing..."
    print "Parsing", args[0], "with", parser.get_parser_name(), parser.get_parser_version()

tm = tm_utils.parse(parser, args[0]) # Parse document by filename

if debug: 
    print "Finished topic map processing..."

# ----------------------------------------------------------------------------
# Printing of statistical information

if test:
    tm_stats = TMStats()
    tm_stats.print_tm_information(tm)

# ----------------------------------------------------------------------------
# Development stuff (ignore)

if grove:
    # Export Topic Map
    from xml.arch import xmlarch
    e = TMExporter()
    e.export(tm, xmlarch.Prettifier(writer=open("export.xml", "w")))

    # Pickle Topic Map
    import pickle
    pickle.dump(tm, open("tm.pic", "w"))

    if 1:
	import pydoc
	pydoc.replace(open("intro.html"), open("intro_tm.html", "w"), globals(), locals())

#import tmtest
#tmtest.test(tm, tm_utils)
